/*
This code demonstrates the PWM control and RPM reading of one of these fans:

http://shop.boxtec.ch/nidec-ta450dc-fan-120x120x38mm-12vdc-two-ball-bearing-p-42482.html
or
http://shop.boxtec.ch/delta-afc1212de-fan-120x120x38mm-12vdc-two-ball-bearing-p-42483.html

Power your Arduino with a 12V power supply via VIN or barrel jack and 
connect the fan as follows:

FAN --- ARDUINO
12V       VIN
GND     GND
PWM     Pin9
RPM      Pin2

2016 BOXTEC AG
http://boxtec.ch
*/

volatile byte half_revolutions;
unsigned int rpm;
unsigned long timeold;
int pwmpin = 9;
int rpmpin = 2;
int fanspeed = 0;  
int speedincr = 1;    

void setup()  { 
  pinMode(pwmpin, OUTPUT);
  // enable pull up
  digitalWrite(rpmpin, HIGH);
  attachInterrupt(0, rpm_fun, RISING);

  half_revolutions = 0;
  rpm = 0;
  timeold = 0;
  Serial.begin(9600);
  Serial.println("Starting..");
}

void loop()  { 
  analogWrite(pwmpin, fanspeed);    

  // change the brightness for next time through the loop:
  fanspeed = fanspeed + speedincr;

  // reverse the direction of the fading at the ends of the fade: 
  if (fanspeed == 0 || fanspeed == 255) {
    speedincr = -speedincr ; 
  }     
  // wait for 30 milliseconds to see the dimming effect    
     //Update RPM every 20 counts, increase this for better RPM resolution,
     //decrease for faster update
  rpm = 30*1000/(millis() - timeold)*half_revolutions;
  timeold = millis();
  half_revolutions = 0;
  Serial.println(rpm,DEC);
 delay(500);
 }

void rpm_fun()
 {
  half_revolutions++;
  //Each rotation, this interrupt function is run twice
 }
